﻿/*
   Copyright 2011 Softisis

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

using System;
using FOM.Interfaces;

namespace FOM.Impl.Conditions
{
    /// <summary>
    /// Simple condition. If source object is not a value type OR a string, it returns true.
    /// </summary>
    public class IsObjectCond:ICondition
    {
        /// <summary>
        /// Returns true if object (and not string)
        /// </summary>
        /// <param name="ctx">The mapping context, with a bunch of info inside.</param>
        /// <returns>true / false</returns>
        public bool CanApplyMapping(MappingContext ctx)
        {
            if (ctx.ObjectSrc==null)
                return true;

            if (ctx.ObjectSrc.GetType().IsValueType || typeof(string).Equals(ctx.ObjectSrc.GetType()))
                return false;

            return true;
        }

        public virtual bool IgnoreChildMappings
        {
            get { return false; }
        }

        public ICondition Reverse()
        {
            return this;
        }
    }
}
