﻿/*
   Copyright 2011 Softisis

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FOM.Interfaces;

namespace FOM.Impl.Conditions
{
    /// <summary>
    /// Object is not processed if found in cache.
    /// Children mappings are not processed.
    /// </summary>
    public class NotInCacheCond : ICondition
    {

        public bool CanApplyMapping(MappingContext ctx)
        {
            if (ctx.ObjectsCache==null)
                return true;
            return !ctx.ObjectsCache.ContainsKey(ctx.ObjectSrc);
        }

        /// <summary>
        /// We don't go on if the object has been in cache
        /// </summary>
        public bool IgnoreChildMappings
        {
            get { return true; }
        }

        public ICondition Reverse()
        {
            return this;
        }
    }
}
