﻿/*
   Copyright 2011 Softisis

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

using System;
using FOM.Interfaces;

namespace FOM.Impl
{

    /// <summary>
    /// Path link implementation.
    /// </summary>
    /// <remarks>Copyright Softisis 2011 (France)</remarks>
    public class PathLink:IPathLink, IComparable<IPathLink>
    {
        private readonly string _srcPath;
        private readonly string _dstPath;

        private readonly string[] _srcPathElts;
        private readonly string[] _dstPathElts;

        public PathLink(string srcPath, string dstPath)
        {
            _srcPath = srcPath;
            _dstPath = dstPath;

            if (_srcPath != null)
                _srcPathElts = _srcPath.Split('.');

            if (dstPath != null)
                _dstPathElts = dstPath.Split('.');
        }

        public string SourcePath
        {
            get { return _srcPath; }
        }

        public string TargetPath
        {
            get { return _dstPath; }
        }

        public string[] SourcePathElements
        {
            get { return _srcPathElts; }
        }

        public string[] TargetPathElements
        {
            get { return _dstPathElts; }
        }

        public IPathLink Reverse()
        {
            return new PathLink(_dstPath, _srcPath);
        }

        public int CompareTo(IPathLink other)
        {
            return TargetPath.CompareTo(other.TargetPath);
        }

        public override bool Equals(object o)
        {
            return Equals((PathLink)o);
        }

        public bool Equals(PathLink other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return Equals(other.TargetPath, TargetPath);
        }

        public override int GetHashCode()
        {
            return (TargetPath != null ? TargetPath.GetHashCode() : 0);
        }
    }
}
